/**
* Library for HC SR04 ultrasonic sensor.
* Original code from http://www.instructables.com/id/Non-blocking-Ultrasonic-Sensor-for-Arduino/
*/

#include "HC_SR04.h"

HC_SR04 *HC_SR04::_instance(NULL);

HC_SR04::HC_SR04(int trigger, int echo)
    : _trigger(trigger), _echo(echo), _int(digitalPinToInterrupt(echo)), _finished(false)
{
  if(_instance==0) _instance=this;    
}

void HC_SR04::begin(){
  pinMode(_trigger, OUTPUT);
  digitalWrite(_trigger, LOW);
  pinMode(_echo, INPUT);  
  attachInterrupt(_int, _echo_isr, CHANGE);
}

void HC_SR04::start(){
  _finished=false;
  digitalWrite(_trigger, HIGH);
  delayMicroseconds(10);
  digitalWrite(_trigger, LOW);  
}

unsigned int HC_SR04::getRange(unsigned int duration, bool unit){
  return duration/((unit == CM) ? DURATION_TO_CM : DURATION_TO_INCH);
}

unsigned int HC_SR04::getRange(bool unit){
  return getRange((_end-_start), unit);
}

unsigned int HC_SR04::getDuration(){
  return (micros() - _start);
}

void HC_SR04::_echo_isr(){
  HC_SR04* _this=HC_SR04::instance();
  
  switch(digitalRead(_this->_echo)){
    case HIGH:
      _this->_start=micros();
      break;
    case LOW:
      _this->_end=micros();
      _this->_finished=true;
      break;
  }   
}
